@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -65,6 +65,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdbool.h>
 
 #include "clamav-types.h"
 #include "clamav-version.h"
@@ -424,7 +425,7 @@ extern cl_error_t cl_engine_settings_free(struct cl_settings *settings);
 /**
  * @brief Prepare the scanning engine.
  *
- * Called this after all required databases have been loaded and settings have
+ * Call this after all required databases have been loaded and settings have
  * been applied.
  *
  * @param engine        A scan engine.
@@ -488,6 +489,52 @@ typedef cl_error_t (*clcb_pre_cache)(int fd, const char *type, void *context);
  */
 extern void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cache callback);
 
+/*
+ * Attributes of each layer in scan.
+ */
+#define LAYER_ATTRIBUTES_NONE 0x0
+#define LAYER_ATTRIBUTES_NORMALIZED 0x1 /** This layer was modified to make matching more generic, reliable. */
+#define LAYER_ATTRIBUTES_DECRYPTED 0x2  /** Decryption was used to extract this layer. I.e. had to decrypt some previous layer. */
+
+/**
+ * @brief File inspection callback.
+ *
+ * DISCLAIMER: This interface is to be considered unstable while we continue to evaluate it.
+ * We may change this interface in the future.
+ *
+ * Called for each NEW file (inner and outer).
+ * Provides capability to record embedded file information during a scan.
+ *
+ * @param fd                  Current file descriptor which is about to be scanned.
+ * @param type                Current file type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE").
+ * @param ancestors           An array of ancestors filenames of size `recursion_level`. filenames may be NULL.
+ * @param parent_file_size    Parent file size.
+ * @param file_name           Current file name, or NULL if the file does not have a name or ClamAV failed to record the name.
+ * @param file_size           Current file size.
+ * @param file_buffer         Current file buffer pointer.
+ * @param recursion_level     Recursion level / depth of the current file.
+ * @param layer_attributes    See LAYER_ATTRIBUTES_* flags.
+ * @param context             Opaque application provided data.
+ * @return                    CL_CLEAN = File is scanned.
+ * @return                    CL_BREAK = Whitelisted by callback - file is skipped and marked as clean.
+ * @return                    CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected.
+ */
+typedef cl_error_t (*clcb_file_inspection)(int fd, const char *type, const char **ancestors, size_t parent_file_size,
+                                           const char *file_name, size_t file_size, const char *file_buffer,
+                                           uint32_t recursion_level, uint32_t layer_attributes, void *context);
+/**
+ * @brief Set a custom file inspection callback function.
+ *
+ * DISCLAIMER: This interface is to be considered unstable while we continue to evaluate it.
+ * We may change this interface in the future.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
+ * @param engine    The initialized scanning engine.
+ * @param callback  The callback function pointer.
+ */
+extern void cl_engine_set_clcb_file_inspection(struct cl_engine *engine, clcb_file_inspection callback);
+
 /**
  * @brief Pre-scan callback.
  *
@@ -1073,6 +1120,18 @@ extern cl_error_t cl_cvdverify(const char *file);
  */
 extern void cl_cvdfree(struct cl_cvd *cvd);
 
+/**
+ * @brief Unpack a CVD file.
+ *
+ * Will verify the CVD is correctly signed unless the `dont_verify` parameter is true.
+ *
+ * @param file          Filepath of CVD file.
+ * @param dir           Destination directory.
+ * @param dont_verify   If true, don't verify the CVD.
+ * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error code.
+ */
+extern cl_error_t cl_cvdunpack(const char *file, const char *dir, bool dont_verify);
+
 /* ----------------------------------------------------------------------------
  * DB directory stat functions.
  * Use these functions to watch for database changes.
@@ -1147,7 +1206,7 @@ extern const char *cl_retver(void);
 /* ----------------------------------------------------------------------------
  * Others.
  */
-extern const char *cl_strerror(int clerror);
+extern const char *cl_strerror(cl_error_t clerror);
 
 /* ----------------------------------------------------------------------------
  * Custom data scanning.
